@@ -8,8 +8,7 @@ import (
 	"k8s.io/apimachinery/pkg/util/intstr"
 
 	"github.com/golang/glog"
-	mapiv1 "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
-	mhcv1beta1 "github.com/openshift/machine-api-operator/pkg/apis/healthchecking/v1beta1"
+	mapiv1 "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 	"github.com/openshift/machine-api-operator/pkg/util/conditions"
 	corev1 "k8s.io/api/core/v1"
 	apimachineryerrors "k8s.io/apimachinery/pkg/api/errors"
@@ -18,6 +17,7 @@ import (
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
 	apimachineryutilerrors "k8s.io/apimachinery/pkg/util/errors"
+	"k8s.io/client-go/tools/record"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
@@ -28,16 +28,41 @@ import (
 
 const (
 	machineAnnotationKey          = "machine.openshift.io/machine"
-	machineRebootAnnotationKey    = "healthchecking.openshift.io/machine-remediation-reboot"
+	machineExternalAnnotationKey  = "host.metal3.io/external-remediation"
 	ownerControllerKind           = "MachineSet"
 	nodeMasterLabel               = "node-role.kubernetes.io/master"
 	machineRoleLabel              = "machine.openshift.io/cluster-api-machine-role"
 	machineMasterRole             = "master"
 	machinePhaseFailed            = "Failed"
-	remediationStrategyAnnotation = "healthchecking.openshift.io/strategy"
-	remediationStrategyReboot     = mhcv1beta1.RemediationStrategyType("reboot")
+	remediationStrategyAnnotation = "machine.openshift.io/remediation-strategy"
+	remediationStrategyExternal   = mapiv1.RemediationStrategyType("external-baremetal")
 	timeoutForMachineToHaveNode   = 10 * time.Minute
 	machineNodeNameIndex          = "machineNodeNameIndex"
+	controllerName                = "machinehealthcheck-controller"
+
+	// Event types
+	// EventRemediationRestricted is emitted in case when machine remediation
+	// is restricted by remediation circuit shorting logic
+	EventRemediationRestricted string = "RemediationRestricted"
+	// EventDetectedUnhealthy is emitted in case a node asociated with a
+	// machine was detected unhealthy
+	EventDetectedUnhealthy string = "DetectedUnhealthy"
+	// EventSkippedMaster is emitted in case an unhealthy node (or a machine
+	// associated with the node) has Master role and external remediation strategy
+	// is not enabled
+	EventSkippedMaster string = "SkippedMaster"
+	// EventMachineDeletionFailed is emitted in case remediation of a machine
+	// is required but deletion of its Machine object failed
+	EventMachineDeletionFailed string = "MachineDeletionFailed"
+	// EventMachineDeleted is emitted when machine was successfully remediated
+	// by deleting its Machine object
+	EventMachineDeleted string = "MachineDeleted"
+	// EventExternalAnnotationFailed is emitted in case adding external annotation
+	// to a Node object failed
+	EventExternalAnnotationFailed string = "ExternalAnnotationFailed"
+	// EventExternalAnnotationAdded is emitted when external annotation was
+	// successfully added to a Node object
+	EventExternalAnnotationAdded string = "ExternalAnnotationAdded"
 )
 
 // Add creates a new MachineHealthCheck Controller and adds it to the Manager. The Manager will set fields on the Controller
@@ -63,6 +88,7 @@ func newReconciler(mgr manager.Manager, opts manager.Options) (*ReconcileMachine
 		client:    mgr.GetClient(),
 		scheme:    mgr.GetScheme(),
 		namespace: opts.Namespace,
+		recorder:  mgr.GetEventRecorderFor(controllerName),
 	}, nil
 }
 
@@ -82,12 +108,12 @@ func indexMachineByNodeName(object runtime.Object) []string {
 
 // add adds a new Controller to mgr with r as the reconcile.Reconciler
 func add(mgr manager.Manager, r reconcile.Reconciler, mapMachineToMHC, mapNodeToMHC handler.ToRequestsFunc) error {
-	c, err := controller.New("machinehealthcheck-controller", mgr, controller.Options{Reconciler: r})
+	c, err := controller.New(controllerName, mgr, controller.Options{Reconciler: r})
 	if err != nil {
 		return err
 	}
 
-	err = c.Watch(&source.Kind{Type: &mhcv1beta1.MachineHealthCheck{}}, &handler.EnqueueRequestForObject{})
+	err = c.Watch(&source.Kind{Type: &mapiv1.MachineHealthCheck{}}, &handler.EnqueueRequestForObject{})
 	if err != nil {
 		return err
 	}
@@ -109,19 +135,20 @@ type ReconcileMachineHealthCheck struct {
 	client    client.Client
 	scheme    *runtime.Scheme
 	namespace string
+	recorder  record.EventRecorder
 }
 
 type target struct {
 	Machine mapiv1.Machine
 	Node    *corev1.Node
-	MHC     mhcv1beta1.MachineHealthCheck
+	MHC     mapiv1.MachineHealthCheck
 }
 
 // Reconcile fetch all targets for a MachineHealthCheck request and does health checking for each of them
 func (r *ReconcileMachineHealthCheck) Reconcile(request reconcile.Request) (reconcile.Result, error) {
 	glog.Infof("Reconciling %s", request.String())
 
-	mhc := &mhcv1beta1.MachineHealthCheck{}
+	mhc := &mapiv1.MachineHealthCheck{}
 	if err := r.client.Get(context.TODO(), request.NamespacedName, mhc); err != nil {
 		if apimachineryerrors.IsNotFound(err) {
 			// Request object not found, could have been deleted after reconcile request.
@@ -140,7 +167,7 @@ func (r *ReconcileMachineHealthCheck) Reconcile(request reconcile.Request) (reco
 	totalTargets := len(targets)
 
 	// health check all targets and reconcile mhc status
-	currentHealthy, needRemediationTargets, nextCheckTimes, errList := healthCheckTargets(targets)
+	currentHealthy, needRemediationTargets, nextCheckTimes, errList := r.healthCheckTargets(targets)
 	if err := r.reconcileStatus(mhc, totalTargets, currentHealthy); err != nil {
 		glog.Errorf("Reconciling %s: error patching status: %v", request.String(), err)
 		return reconcile.Result{}, err
@@ -154,6 +181,15 @@ func (r *ReconcileMachineHealthCheck) Reconcile(request reconcile.Request) (reco
 			mhc.Spec.MaxUnhealthy,
 			totalTargets-currentHealthy,
 		)
+		r.recorder.Eventf(
+			mhc,
+			corev1.EventTypeWarning,
+			EventRemediationRestricted,
+			"Remediation restricted due to exceeded number of unhealthy machines (total: %v, unhealthy: %v, maxUnhealthy: %v)",
+			totalTargets,
+			totalTargets-currentHealthy,
+			mhc.Spec.MaxUnhealthy,
+		)
 		return reconcile.Result{Requeue: true}, nil
 	}
 	glog.V(3).Infof("Reconciling %s: monitoring MHC: total targets: %v,  maxUnhealthy: %v, unhealthy: %v. Remediations are allowed",
@@ -188,7 +224,7 @@ func (r *ReconcileMachineHealthCheck) Reconcile(request reconcile.Request) (reco
 	return reconcile.Result{}, nil
 }
 
-func isAllowedRemediation(mhc *mhcv1beta1.MachineHealthCheck) bool {
+func isAllowedRemediation(mhc *mapiv1.MachineHealthCheck) bool {
 	if mhc.Spec.MaxUnhealthy == nil {
 		return true
 	}
@@ -210,10 +246,11 @@ func derefInt(i *int) int {
 	return 0
 }
 
-func (r *ReconcileMachineHealthCheck) reconcileStatus(mhc *mhcv1beta1.MachineHealthCheck, targets, currentHealthy int) error {
+func (r *ReconcileMachineHealthCheck) reconcileStatus(mhc *mapiv1.MachineHealthCheck, targets, currentHealthy int) error {
 	baseToPatch := client.MergeFrom(mhc.DeepCopy())
 	mhc.Status.ExpectedMachines = &targets
 	mhc.Status.CurrentHealthy = &currentHealthy
+
 	if err := r.client.Status().Patch(context.Background(), mhc, baseToPatch); err != nil {
 		return err
 	}
@@ -222,7 +259,7 @@ func (r *ReconcileMachineHealthCheck) reconcileStatus(mhc *mhcv1beta1.MachineHea
 
 // healthCheckTargets health checks a slice of targets
 // and gives a data to measure the average health
-func healthCheckTargets(targets []target) (int, []target, []time.Duration, []error) {
+func (r *ReconcileMachineHealthCheck) healthCheckTargets(targets []target) (int, []target, []time.Duration, []error) {
 	var nextCheckTimes []time.Duration
 	var errList []error
 	var needRemediationTargets []target
@@ -243,6 +280,14 @@ func healthCheckTargets(targets []target) (int, []target, []time.Duration, []err
 
 		if nextCheck > 0 {
 			glog.V(3).Infof("Reconciling %s: is likely to go unhealthy in %v", t.string(), nextCheck)
+			r.recorder.Eventf(
+				&t.Machine,
+				corev1.EventTypeNormal,
+				EventDetectedUnhealthy,
+				"Machine %v has unhealthy node %v",
+				t.string(),
+				t.nodeName(),
+			)
 			nextCheckTimes = append(nextCheckTimes, nextCheck)
 			continue
 		}
@@ -254,7 +299,7 @@ func healthCheckTargets(targets []target) (int, []target, []time.Duration, []err
 	return currentHealthy, needRemediationTargets, nextCheckTimes, errList
 }
 
-func (r *ReconcileMachineHealthCheck) getTargetsFromMHC(mhc mhcv1beta1.MachineHealthCheck) ([]target, error) {
+func (r *ReconcileMachineHealthCheck) getTargetsFromMHC(mhc mapiv1.MachineHealthCheck) ([]target, error) {
 	machines, err := r.getMachinesFromMHC(mhc)
 	if err != nil {
 		return nil, fmt.Errorf("error getting machines from MHC: %v", err)
@@ -284,7 +329,7 @@ func (r *ReconcileMachineHealthCheck) getTargetsFromMHC(mhc mhcv1beta1.MachineHe
 	return targets, nil
 }
 
-func (r *ReconcileMachineHealthCheck) getMachinesFromMHC(mhc mhcv1beta1.MachineHealthCheck) ([]mapiv1.Machine, error) {
+func (r *ReconcileMachineHealthCheck) getMachinesFromMHC(mhc mapiv1.MachineHealthCheck) ([]mapiv1.Machine, error) {
 	selector, err := metav1.LabelSelectorAsSelector(&mhc.Spec.Selector)
 	if err != nil {
 		return nil, fmt.Errorf("failed to build selector")
@@ -334,7 +379,7 @@ func (r *ReconcileMachineHealthCheck) mhcRequestsFromNode(o handler.MapObject) [
 		return nil
 	}
 
-	mhcList := &mhcv1beta1.MachineHealthCheckList{}
+	mhcList := &mapiv1.MachineHealthCheckList{}
 	if err := r.client.List(context.Background(), mhcList); err != nil {
 		glog.Errorf("No-op: Unable to list mhc: %v", err)
 		return nil
@@ -364,7 +409,7 @@ func (r *ReconcileMachineHealthCheck) mhcRequestsFromMachine(o handler.MapObject
 		return nil
 	}
 
-	mhcList := &mhcv1beta1.MachineHealthCheckList{}
+	mhcList := &mapiv1.MachineHealthCheckList{}
 	if err := r.client.List(context.Background(), mhcList); err != nil {
 		glog.Errorf("No-op: Unable to list mhc: %v", err)
 		return nil
@@ -388,37 +433,74 @@ func (t *target) remediate(r *ReconcileMachineHealthCheck) error {
 
 	remediationStrategy, ok := t.MHC.Annotations[remediationStrategyAnnotation]
 	if ok {
-		if mhcv1beta1.RemediationStrategyType(remediationStrategy) == remediationStrategyReboot {
-			return t.remediationStrategyReboot(r)
+		if mapiv1.RemediationStrategyType(remediationStrategy) == remediationStrategyExternal {
+			return t.remediationStrategyExternal(r)
 		}
 	}
 	if t.isMaster() {
+		r.recorder.Eventf(
+			&t.Machine,
+			corev1.EventTypeNormal,
+			EventSkippedMaster,
+			"Machine %v is a master node, skipping remediation",
+			t.string(),
+		)
 		glog.Infof("%s: master node, skipping remediation", t.string())
 		return nil
 	}
 
 	glog.Infof("%s: deleting", t.string())
 	if err := r.client.Delete(context.TODO(), &t.Machine); err != nil {
+		r.recorder.Eventf(
+			&t.Machine,
+			corev1.EventTypeWarning,
+			EventMachineDeletionFailed,
+			"Machine %v remediation failed: unable to delete Machine object: %v",
+			t.string(),
+			err,
+		)
 		return fmt.Errorf("%s: failed to delete machine: %v", t.string(), err)
 	}
+	r.recorder.Eventf(
+		&t.Machine,
+		corev1.EventTypeNormal,
+		EventMachineDeleted,
+		"Machine %v has been remediated by requesting to delete Machine object",
+		t.string(),
+	)
 	return nil
 }
 
-func (t *target) remediationStrategyReboot(r *ReconcileMachineHealthCheck) error {
-	// we already have reboot annotation on the node, stop reconcile
-	if _, ok := t.Node.Annotations[machineRebootAnnotationKey]; ok {
+func (t *target) remediationStrategyExternal(r *ReconcileMachineHealthCheck) error {
+	// we already have external annotation on the machine, stop reconcile
+	if _, ok := t.Machine.Annotations[machineExternalAnnotationKey]; ok {
 		return nil
 	}
 
-	if t.Node.Annotations == nil {
-		t.Node.Annotations = map[string]string{}
+	if t.Machine.Annotations == nil {
+		t.Machine.Annotations = map[string]string{}
 	}
 
-	glog.Infof("Machine %s has been unhealthy for too long, adding reboot annotation", t.Machine.Name)
-	t.Node.Annotations[machineRebootAnnotationKey] = ""
-	if err := r.client.Update(context.TODO(), t.Node); err != nil {
+	glog.Infof("Machine %s has been unhealthy for too long, adding external annotation", t.Machine.Name)
+	t.Machine.Annotations[machineExternalAnnotationKey] = ""
+	if err := r.client.Update(context.TODO(), &t.Machine); err != nil {
+		r.recorder.Eventf(
+			&t.Machine,
+			corev1.EventTypeWarning,
+			EventExternalAnnotationFailed,
+			"Requesting external remediation of node associated with machine %v failed: %v",
+			t.string(),
+			err,
+		)
 		return err
 	}
+	r.recorder.Eventf(
+		&t.Machine,
+		corev1.EventTypeNormal,
+		EventExternalAnnotationAdded,
+		"Requesting external remediation of node associated with machine %v",
+		t.string(),
+	)
 	return nil
 }
 
@@ -574,7 +656,7 @@ func namespacedName(obj metav1.Object) types.NamespacedName {
 	}
 }
 
-func hasMatchingLabels(machineHealthCheck *mhcv1beta1.MachineHealthCheck, machine *mapiv1.Machine) bool {
+func hasMatchingLabels(machineHealthCheck *mapiv1.MachineHealthCheck, machine *mapiv1.Machine) bool {
 	selector, err := metav1.LabelSelectorAsSelector(&machineHealthCheck.Spec.Selector)
 	if err != nil {
 		glog.Warningf("unable to convert selector: %v", err)
