@@ -74,13 +74,19 @@ func (optr *Operator) syncClusterAPIController(config *OperatorConfig) error {
 }
 
 func (optr *Operator) syncBaremetalControllers(config *OperatorConfig) error {
-	// First create a Secret needed for the Metal3 deployment
+	// Try to get baremetal provisioning config from a CR
+	baremetalProvisioningConfig, err := getBaremetalProvisioningConfig(optr.dynamicClient, baremetalProvisioningCR)
+	if err != nil {
+		glog.Errorf("Unable to read Baremetal Provisioning config from CR %s.", baremetalProvisioningCR)
+		glog.Infof("Will try to read Baremetal Provisioning config from ConfigMap %s instead", baremetalConfigmap)
+	}
+	// Create a Secret needed for the Metal3 deployment
 	if err := createMariadbPasswordSecret(optr.kubeClient.CoreV1(), config); err != nil {
 		glog.Error("Not proceeding with Metal3 deployment. Failed to create Mariadb password.")
 		return err
 	}
 
-	metal3Deployment := newMetal3Deployment(config)
+	metal3Deployment := newMetal3Deployment(config, baremetalProvisioningConfig)
 	_, updated, err := resourceapply.ApplyDeployment(optr.kubeClient.AppsV1(), metal3Deployment)
 	if err != nil {
 		return err
@@ -188,9 +194,13 @@ func newPodTemplateSpec(config *OperatorConfig, features map[string]bool) *corev
 			},
 		},
 		Spec: corev1.PodSpec{
-			Containers:         containers,
-			PriorityClassName:  "system-node-critical",
-			NodeSelector:       map[string]string{"node-role.kubernetes.io/master": ""},
+			Containers:        containers,
+			PriorityClassName: "system-node-critical",
+			NodeSelector:      map[string]string{"node-role.kubernetes.io/master": ""},
+			SecurityContext: &corev1.PodSecurityContext{
+				RunAsNonRoot: pointer.BoolPtr(true),
+				RunAsUser:    pointer.Int64Ptr(65534),
+			},
 			ServiceAccountName: "machine-api-controllers",
 			Tolerations:        tolerations,
 		},
@@ -219,10 +229,11 @@ func newContainers(config *OperatorConfig, features map[string]bool) []corev1.Co
 			Resources: resources,
 		},
 		{
-			Name:    "machine-controller",
-			Image:   config.Controllers.Provider,
-			Command: []string{"/machine-controller-manager"},
-			Args:    args,
+			Name:      "machine-controller",
+			Image:     config.Controllers.Provider,
+			Command:   []string{"/machine-controller-manager"},
+			Args:      args,
+			Resources: resources,
 			Env: []corev1.EnvVar{
 				{
 					Name: "NODE_NAME",
